﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectRepairWork.Entities;
using ProjectRepairWork.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectRepairWork.Repositories.Implementation;
public class PremisesRepository : IPremisesRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<PremisesRepository> _logger;

    public PremisesRepository(IConnectionString connectionString, ILogger<PremisesRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreatePremises(Premises premises)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(premises));

        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);

            var queryInsert = @"
INSERT INTO Premises (ClientName, Address, Area, DateStart, DateEnd, PremisesType)
VALUES (@ClientName,@Address,@Area,@DateStart,@DateEnd,@PremisesType)";
            connection.Execute(queryInsert, premises);
        }

        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public void DeletePremises(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"
                DELETE FROM Premises
                WHERE Id=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public IEnumerable<Premises> ReadPremises()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM Premises";
            var premises = connection.Query<Premises>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(premises));
            return premises;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

    public Premises ReadPremisesById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
                SELECT * FROM Premises
                WHERE Id=@id";
            var premises = connection.QueryFirst<Premises>(querySelect, new { id });
            _logger.LogDebug("Найденный объект: {json}",
                JsonConvert.SerializeObject(premises));
            return premises;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

    public void UpdatePremises(Premises premises)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}",
            JsonConvert.SerializeObject(premises));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"
                UPDATE Premises
                SET
                ClientName = @ClientName, 
                Address = @Address,
                Area = @Area, 
                DateStart = @DateStart, 
                DateEnd = @DateEnd, 
                PremisesType = @PremisesType 
                WHERE Id=@Id";
            connection.Execute(queryUpdate, premises);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }
}
